@@ -22,12 +22,12 @@
  */
 
 #include <stdlib.h>
-#include <syslog.h>
 #include <gtk/gtk.h>
 
 #include "config.h"
 
 #include "gdm.h"
+#include "gdmcommon.h"
 #include "gdmcomm.h"
 #include "gdmconfig.h"
 
@@ -37,19 +37,7 @@ static GHashTable *int_hash       = NULL;
 static GHashTable *bool_hash      = NULL;
 static GHashTable *string_hash    = NULL;
 static gboolean gdm_never_cache   = FALSE;
-
-/*
- * Hack to keep track if config functions should be printing error messages
- * to syslog or stdout
- */
-static gboolean using_syslog = FALSE;
-
-void
-gdm_openlog (const char *ident, int logopt, int facility)
-{
-   openlog (ident, logopt, facility);
-   using_syslog = TRUE;
-}
+static int comm_tries             = 5;
 
 /**
  * gdm_config_never_cache
@@ -69,6 +57,18 @@ gdm_config_never_cache (gboolean never_cache)
    gdm_never_cache = never_cache;
 }
 
+/**
+ * gdm_config_set_comm_retries
+ *
+ * If a client wants to specify how many times it will retry to
+ * get a config value, this function can be used.
+ */
+void
+gdm_config_set_comm_retries (int tries)
+{
+    comm_tries = tries;
+}
+
 /**
  * gdm_config_hash_lookup
  *
@@ -109,6 +109,7 @@ gdm_config_add_hash (GHashTable *hash, gchar *key, gpointer value)
 	if (p != NULL)
 		*p = '\0';
 
+	/* Do not free the newkey since it is going into the hash. */
 	g_hash_table_insert (hash, newkey, value);
 }
 
@@ -133,14 +134,13 @@ gdm_config_get_result (gchar *key)
 
 	command = g_strdup_printf ("GET_CONFIG %s", newkey);
 	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
-		"2.13.0.1", 5);
+		"2.13.0.1", comm_tries);
 
 	g_free (command);
 	g_free (newkey);
 	return result;
 }
 
-
 /**
  * gdm_config_get_xserver_details
  *
@@ -155,17 +155,14 @@ gdm_config_get_xserver_details (gchar *xserver, gchar *key)
 
 	command = g_strdup_printf ("GET_SERVER_DETAILS %s %s", xserver, key);
 	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
-		"2.13.0.1", 5);
+		"2.13.0.1", comm_tries);
 
 	g_free (command);
 
 	if (! result || ve_string_empty (result) ||
 	    strncmp (result, "OK ", 3) != 0) {
 
-		if (using_syslog)
-			syslog (LOG_ERR, "Could not access xserver configuration");
-		else
-			printf ("Could not access xserver configuration");
+		gdm_common_error ("Could not access xserver configuration");
 
 		if (result)
 			g_free (result);
@@ -196,17 +193,14 @@ gdm_config_get_xservers (gboolean flexible)
 
 	command = g_strdup_printf ("GET_SERVER_LIST");
 	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
-		"2.13.0.1", 5);
+		"2.13.0.1", comm_tries);
 
 	g_free (command);
 
 	if (! result || ve_string_empty (result) ||
 	    strncmp (result, "OK ", 3) != 0) {
 
-		if (using_syslog)
-			syslog (LOG_ERR, "Could not access xserver configuration");
-		else
-			printf ("Could not access xserver configuration");
+		gdm_common_error ("Could not access xserver configuration");
 
 		if (result)
 			g_free (result);
@@ -308,9 +302,9 @@ gdm_config_get_xservers (gboolean flexible)
  * access is faster.
  */
 static gchar *
-_gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean show_error)
+_gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean doing_translated)
 {
-	gchar **hashretval = NULL;
+	gchar *hashretval = NULL;
 	gchar *result = NULL;
 	gchar *temp;
 
@@ -320,58 +314,69 @@ _gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean
 	hashretval = gdm_config_hash_lookup (string_hash, key);
 
 	if (reload == FALSE && hashretval != NULL)
-		return *hashretval;
+		return hashretval;
 
 	result = gdm_config_get_result (key);
 
-	if (! result || ve_string_empty (result) ||
+	if ( ! result || ve_string_empty (result) ||
 	    strncmp (result, "OK ", 3) != 0) {
 
-		if (show_error) {
-			if (using_syslog)
-				syslog (LOG_ERR, "Could not access configuration key %s", key);
-			else
-				printf ("Could not access configuration key %s\n", key);
+		gchar *getdefault;
+
+		/*
+		 * If looking for a translated string, and not found, just return
+		 * NULL.
+		 */
+		if (doing_translated) {
+			if (result)
+				g_free (result);
+			return NULL;
 		}
 
-		if (result)
-			g_free (result);
-		return NULL;
+		gdm_common_error ("Could not access configuration key <%s>", key);
+
+		/* Return the compiled in value associated with the key, if available. */
+		getdefault = strchr (key, '=');
+		if (getdefault != NULL)
+			getdefault++;
+
+		temp = g_strdup (getdefault);
+
+		gdm_common_error ("Using compiled in value <%s> for <%s>", temp, key);
+	} else {
+
+		/* skip the "OK " */
+		temp = g_strdup (result + 3);
 	}
 
-	/* skip the "OK " */
-	temp = g_strdup (result + 3);
-	g_free (result);
+	if (result)
+		g_free (result);
 
 	if (hashretval == NULL) {
-		gchar **charval = g_new0 (gchar *, 1);
-		*charval = temp;
-		gdm_config_add_hash (string_hash, key, charval);
 
 		if (changed != NULL)
 			*changed = TRUE;
 
-		return *charval;
+		gdm_config_add_hash (string_hash, key, temp);
 	} else {
 		if (changed != NULL) {
-			if (strcmp (ve_sure_string (*hashretval), temp) != 0)
+			if (strcmp (ve_sure_string (hashretval), temp) != 0)
 				*changed = TRUE;
 			else
 				*changed = FALSE;
 		}
-
-		*hashretval = temp;
-		return *hashretval;
+		g_hash_table_replace (string_hash, key, temp);
 	}
+	return temp;
 }
 
 gchar *
 gdm_config_get_string (gchar *key)
 {
    if (gdm_never_cache == TRUE)
-      return _gdm_config_get_string (key, TRUE, NULL, TRUE);
+      return _gdm_config_get_string (key, TRUE, NULL, FALSE);
    else
-      return _gdm_config_get_string (key, FALSE, NULL, TRUE);
+      return _gdm_config_get_string (key, FALSE, NULL, FALSE);
 }
 
 /**
@@ -388,34 +393,38 @@ static gchar *
 _gdm_config_get_translated_string (gchar *key, gboolean reload, gboolean *changed)
 {
         const GList *li;
-        char *dkey;
+        char *newkey;
         char *def;
 
 	/* Strip key */
-        dkey = g_strdup (key);
-        def = strchr (dkey, '=');
-        if (def != NULL) {
-                *def = '\0';
-                def++;
-        }
+	newkey = g_strdup (key);
+	def    = strchr (newkey, '=');
+	if (def != NULL)
+		*def = '\0';
 
-        for (li = ve_i18n_get_language_list ("LC_MESSAGES");
-             li != NULL;
-             li = li->next) {
-                gchar *full = g_strdup_printf ("%s[%s]", dkey, (char *)li->data);
+	for (li = ve_i18n_get_language_list ("LC_MESSAGES");
+	     li != NULL;
+	     li = li->next) {
+                gchar *full = g_strdup_printf ("%s[%s]", newkey, (char *)li->data);
 
 		/*
-		 * Pass FALSE for last argument so it doesn't print errors for
+		 * Pass TRUE for last argument so it doesn't print errors for
 		 * failing to find the key, since this is expected
 		 */
-	        gchar *val = _gdm_config_get_string (full, reload, changed, FALSE);
+		gchar *val = _gdm_config_get_string (full, reload, changed, TRUE);
 
-                if (val != NULL)
+		g_free (full);
+
+                if (val != NULL) {
+			g_free (newkey);
 			return val;
+		}
         }
 
+	g_free (newkey);
+
 	/* Print error if it fails this time */
-	return _gdm_config_get_string (key, reload, changed, TRUE);
+	return _gdm_config_get_string (key, reload, changed, FALSE);
 }
 
 gchar *
@@ -450,26 +459,34 @@ _gdm_config_get_int (gchar *key, gboolean reload, gboolean *changed)
 
 	result = gdm_config_get_result (key);
 
-	if (! result || ve_string_empty (result) ||
+	if ( ! result || ve_string_empty (result) ||
 	    strncmp (result, "OK ", 3) != 0) {
 
-		if (using_syslog)
-			syslog (LOG_ERR, "Could not access configuration key %s", key);
-		else
-			printf ("Could not access configuration key %s\n", key);
+		gchar *getdefault;
 
-		if (result)
-			g_free (result);
-		return 0;
+		gdm_common_error ("Could not access configuration key <%s>", key);
+
+		/* Return the compiled in value associated with the key, if available. */
+		getdefault = strchr (key, '=');
+		if (getdefault != NULL)
+			getdefault++;
+
+		temp = atoi (getdefault);
+
+		gdm_common_error ("Using compiled in value <%d> for <%s>", temp, key);
+
+	} else {
+
+		/* skip the "OK " */
+		temp = atoi (result + 3);
 	}
 
-	/* skip the "OK " */
-	temp = atoi (result + 3);
-	g_free (result);
+	if (result)
+		g_free (result);
 
 	if (hashretval == NULL) {
 		gint *intval = g_new0 (gint, 1);
-		*intval = temp;
+		*intval      = temp;
 		gdm_config_add_hash (int_hash, key, intval);
 
 		if (changed != NULL)
@@ -521,29 +538,46 @@ _gdm_config_get_bool (gchar *key, gboolean reload, gboolean *changed)
 
 	result = gdm_config_get_result (key);
 
-	if (! result || ve_string_empty (result) ||
+	if ( ! result || ve_string_empty (result) ||
 	    strncmp (result, "OK ", 3) != 0) {
 
-		if (using_syslog)
-			syslog (LOG_ERR, "Could not access configuration key %s", key);
-		else
-			printf ("Could not access configuration key %s\n", key);
+		gchar *getdefault;
 
-		if (result)
-			g_free (result);
-		return FALSE;
+		gdm_common_error ("Could not access configuration key <%s>", key);
+
+		/* Return the compiled in value associated with the key, if available. */
+		getdefault = strchr (key, '=');
+		if (getdefault != NULL)
+			getdefault++;
+
+		/* Same logic as used in ve_config_get_bool */
+		if (getdefault != NULL &&
+		   (getdefault[0] == 'T' ||
+		    getdefault[0] == 't' ||
+		    getdefault[0] == 'Y' ||
+		    getdefault[0] == 'y' ||
+		    atoi (getdefault) != 0)) {
+			temp = TRUE;
+			gdm_common_error ("Using compiled in value <TRUE> for <%s>", key);
+		} else {
+			temp = FALSE;
+			gdm_common_error ("Using compiled in value <FALSE> for <%s>", key);
+		}
+	} else {
+
+		/* skip the "OK " */
+		if (strcmp (ve_sure_string (result + 3), "true") == 0)
+			temp = TRUE;
+		else
+			temp = FALSE;
 	}
 
-	/* skip the "OK " */
-	if (strcmp (ve_sure_string (result + 3), "true") == 0)
-		temp = TRUE;
-	else
-		temp = FALSE;
-	g_free (result);
+	if (result)
+		g_free (result);
 
 	if (hashretval == NULL) {
 		gboolean *boolval = g_new0 (gboolean, 1);
-		*boolval = temp;
+		*boolval          = temp;
 		gdm_config_add_hash (bool_hash, key, boolval);
 
 		if (changed != NULL)
@@ -585,7 +619,7 @@ gboolean
 gdm_config_reload_string (gchar *key)
 {
 	gboolean changed;
-	_gdm_config_get_string (key, TRUE, &changed, TRUE);
+	_gdm_config_get_string (key, TRUE, &changed, FALSE);
 	return changed;
 }
 
