@@ -61,10 +61,14 @@ bin_exists (const char *command)
 	}
 }
 
+/*
+ * The buttons with these handlers appear in the F10 menu, so they
+ * cannot depend on callback data being passed in.
+ */
 static void
 query_greeter_restart_handler (void)
 {
-	if (gdm_common_warn (_("Are you sure you want to restart the computer?"), "",
+	if (gdm_wm_warn_dialog (_("Are you sure you want to restart the computer?"), "",
 			     _("_Restart"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		closelog ();
 		
@@ -76,7 +80,7 @@ query_greeter_restart_handler (void)
 static void
 query_greeter_halt_handler (void)
 {
-	if (gdm_common_warn (_("Are you sure you want to Shut Down the computer?"), "",
+	if (gdm_wm_warn_dialog (_("Are you sure you want to Shut Down the computer?"), "",
 			     _("Shut _Down"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		closelog ();
 
@@ -87,7 +91,7 @@ query_greeter_halt_handler (void)
 static void
 query_greeter_suspend_handler (void)
 {
-	if (gdm_common_warn (_("Are you sure you want to suspend the computer?"), "",
+	if (gdm_wm_warn_dialog (_("Are you sure you want to suspend the computer?"), "",
 			     _("_Suspend"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		/* suspend interruption */
 		printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_SUSPEND);
@@ -139,16 +143,12 @@ void
 greeter_system_append_system_menu (GtkWidget *menu)
 {
 	GtkWidget *w, *sep;
-	static GtkTooltips *tooltips = NULL;
 
 	/* should never be allowed by the UI */
 	if ( ! gdm_config_get_bool (GDM_KEY_SYSTEM_MENU) ||
 	    ve_string_empty (g_getenv ("GDM_IS_LOCAL")))
 		return;
 
-	if (tooltips == NULL)
-		tooltips = gtk_tooltips_new ();
-
 	if (gdm_config_get_bool (GDM_KEY_CHOOSER_BUTTON)) {
 		w = gtk_menu_item_new_with_mnemonic (_("Remote Login via _XDMCP..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
@@ -156,11 +156,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		g_signal_connect (G_OBJECT (w), "activate",
 				  G_CALLBACK (greeter_chooser_handler),
 				  NULL);
-		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
-				      _("Run an XDMCP chooser which will allow "
-					"you to log into available remote "
-					"computers, if there are any."),
-				      NULL);
 	}
 
 	if (gdm_config_get_bool (GDM_KEY_CONFIG_AVAILABLE) &&
@@ -171,10 +166,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		g_signal_connect (G_OBJECT (w), "activate",
 				  G_CALLBACK (greeter_config_handler),
 				  NULL);
-		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
-				      _("Configure GDM (this login manager). "
-					"This will require the root password."),
-				      NULL);
 	}
 
 	if (GdmRebootFound || GdmHaltFound || GdmSuspendFound) {
@@ -190,9 +181,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		g_signal_connect (G_OBJECT (w), "activate",
 				  G_CALLBACK (query_greeter_restart_handler),
 				  NULL);
-		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
-				      _("Restart your computer"),
-				      NULL);
 	}
 
 	if (GdmHaltFound) {
@@ -202,10 +190,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		g_signal_connect (G_OBJECT (w), "activate",
 				  G_CALLBACK (query_greeter_halt_handler),
 				  NULL);
-		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
-				      _("Shut Down the system so that "
-					"you may safely turn off the computer."),
-				      NULL);
 	}
 
 	if (GdmSuspendFound) {
@@ -215,9 +199,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		g_signal_connect (G_OBJECT (w), "activate",
 				  G_CALLBACK (query_greeter_suspend_handler),
 				  NULL);
-		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
-				      _("Suspend your computer"),
-				      NULL);
 	}
 }
 
